/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2008 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Class
    Foam::colloidParticleCloud

Description
    A Cloud of colloid particles

SourceFiles
    colloidParticleCloudI.H
    colloidParticleCloud.C
    colloidParticleCloudIO.C

\*---------------------------------------------------------------------------*/

#ifndef colloidParticleCloud_H
#define colloidParticleCloud_H

#include "Cloud.H"
#include "colloidParticle.H"
#include "autoPtr.H"
#include "IOdictionary.H"
#include "Random.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// Forward declaration of classes
class fvMesh;

class collisionModel;

/*---------------------------------------------------------------------------*\
                           Class colloidParticleCloud Declaration
\*---------------------------------------------------------------------------*/

class colloidParticleCloud
:
    public Cloud<colloidParticle>
{
    // Private data

        const fvMesh& mesh_;
        const Time& runTime_;
        Random random_;

        IOdictionary particleProperties_;
        IOdictionary transportProperties_;
        autoPtr<collisionModel> collisionModel_;

        scalar rhop_;
        scalar e_;
        scalar mu_;
        bool useSourceMoment_;
        bool inject_;
        word vdwModel_;
        word edlModel_;
        scalar ppts0_;
        scalar ppts1_;
        vector c0_;
        vector c1_;
        scalar vel0_;
        vector vel1_;
        scalar d0_;
        scalar d1_;
        scalar tStart_;
        scalar tEnd_;

        scalar Awall_;
        scalar Apart_;
        scalar psiWall_;
        scalar psiPart_;
        scalar molarity_;
        scalar perm_;
        scalar T_;

        scalar rhoc_;
        scalar nuc_;

        vectorField smoment_;


    // Private Member Functions

        //- Disallow default bitwise copy construct
        colloidParticleCloud(const colloidParticleCloud&);

        //- Disallow default bitwise assignment
        void operator=(const colloidParticleCloud&);


public:

    // Constructors

        //- Construct given mesh
        colloidParticleCloud
        (
            const fvMesh&,
            const word& cloudName = "defaultCloud"
        );


    // Member Functions

        // Access

            inline const fvMesh& mesh() const;

            inline scalar rhop() const;
            inline scalar e() const;
            inline scalar mu() const;

            inline scalar rhoc() const;
            inline scalar nuc() const;

            inline word vdwModel() const;
            inline word edlModel() const;

            inline scalar Awall() const;
            inline scalar Apart() const;
            inline scalar psiWall() const;
            inline scalar psiPart() const;
            inline scalar molarity() const;
            inline scalar perm() const;
            inline scalar T() const;

            inline tmp<volVectorField> momentumSource(const dimensionedScalar& rho) const;
            inline const collisionModel& collisions() const;
            vectorField &smoment() { return smoment_; }

            Random &random() { return random_; }


        // Edit

            //- Move the particles under the influence of the given
            //  gravitational acceleration
            void move(const dimensionedVector& g);

            //- Inject particles
            void inject(colloidParticle::trackData &td);


        // I-O

            //- Write fields
            void writeFields() const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#include "colloidParticleCloudI.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
